import es, playerlib
import time
import vecmath

info = es.AddonInfo()
info.name = 'Script helper script'
info.basename = 'sh'
info.description = 'Provides some helper functions for other scripts'
info.url = 'http://addons.eventscripts.com/addons/view/sh'
info.author = 'GODJonez'
info.version = '1.2'

es.ServerVar('es_sh_v', info.version, '%s version'%info.name).makepublic()

def startTicker():
    es.addons.registerTickListener(_ticker)

def stopTicker():
    es.addons.unregisterTickListener(_ticker)

class Holder(object):
    def __init__(self, index, prop, value, secs):
        self.index = index
        self.prop = prop
        self.value = value
        self.secs = time.time() + secs
    def modify(self, now):
        pass
    def doit(self, now):
        self.modify(now)
        es.setindexprop(self.index, self.prop, str(self.value))
        if now >= self.secs:
            return False
        return True

class Functional(Holder):
    def __init__(self, index, prop, value1, value2, secs, modifier):
        now = time.time()
        self.index = index
        self.prop = prop
        if ',' in str(value1):
            self.value1 = vecmath.vector(value1)
            self.value2 = vecmath.vector(value2)
        else:
            self.value1 = float(value1)
            self.value2 = float(value2)
        self.value = self.value1
        self.secs = now + secs
        self.interval = (self.value2 - self.value1)/secs
        self.modifier = modifier
    def modify(self, now):
        self.value = self.modifier(self.value2 - self.interval*(self.secs-now))

class FunctionalCommand(Functional):
    def __init__(self, value1, value2, secs, modifier, func, pre_args, post_args, kw):
        Functional.__init__(self, 0, "", value1, value2, secs, modifier)
        if not isinstance(pre_args, tuple):
            pre_args = (pre_args,)
        if not isinstance(post_args, tuple):
            post_args = (post_args,)
        if callable(func):
            self.func = func
            self.pre_args = pre_args
            self.post_args = post_args
            self.kw = kw
            self.combine = self.py_combine
        else:
            self.func = es.ServerCommand
            self.pre_args = str(func)+' '+' '.join(pre_args)
            self.post_args = ' '.join(post_args)
            self.kw = None
            self.combine = self.es_combine
        if self.kw == None:
            self.kw = {}
    def py_combine(self):
        return self.pre_args + (self.value,) + self.post_args
    def es_combine(self):
        return (('%s %s'%(self.pre_args, self.post_args)).replace('%%', str(self.value))),
    def doit(self, now):
        self.modify(now)
        args = self.combine()
        self.func(*args, **self.kw)
        if now >= self.secs:
            return False
        return True
        
holders = set()

def _ticker():
    if not holders:
        stopTicker()
        return
    expired = set()
    now = time.time()
    for x in holders:
        if not x.doit(now):
            expired.add(x)
    if expired:
        holders.difference_update(expired)

def holdplayerprop(userid, prop, value, secs):
    index = playerlib.getPlayer(userid).get('index') 
    return holdindexprop(index, prop, value, secs)

def linearplayerprop(userid, prop, value1, value2, secs):
    index = playerlib.getPlayer(userid).get('index')
    return linearplayerprop(index, prop, value1, value2, secs)

def functionalplayerprop(userid, prop, value1, value2, secs, modifier):
    index = playerlib.getPlayer(userid).get('index')
    return functionalindexprop(index, prop, value1, value2, secs, modifier)

def holdindexprop(index, prop, value, secs):
    holders.add(Holder(index,prop,value,secs))
    startTicker()
    return True

def linearindexprop(index, prop, value1, value2, secs):
    return functionalindexprop(index,prop,value1,value2,secs, lambda x:x)

def functionalindexprop(index, prop, value1, value2, secs, modifier):
    holders.add(Functional(index,prop,value1,value2,secs, modifier))
    startTicker()
    return True

def linearcommand(value1, value2, secs, func, pre_args = (), post_args = (), kw = None):
    return functionalcommand(value1, value2, secs, lambda x:x, func, pre_args, post_args, kw)

def functionalcommand(value1, value2, secs, modifier, func, pre_args = (), post_args = (), kw = None):
    holders.add(FunctionalCommand(value1, value2, secs, modifier, func, pre_args, post_args, kw))
    startTicker()
    return True

def getargs(start=0):
    return [es.getargv(x) for x in xrange(start, es.getargc())]

def dbgobj(msg, obj, level=0):
    es.dbgmsg(level, '%s=%s %s@%08x'%(msg,repr(obj),type(obj),id(obj)))
    
